//
//  TTApplication.m
//  DAOApp
//
//  Created by Erlon Cabral on 03/03/12.
//  Copyright (c) 2012 aftertaste.cc. All rights reserved.
//

#import "TTApplication.h"

// controller
#import "TTPeopleViewController.h"

// model
#import "TTModel.h"
#import "TTPerson.h"
#import "TTGender.h"

@interface TTApplication ()
@property (nonatomic, retain) UIViewController *rootViewController;
@end

@implementation TTApplication

@synthesize rootViewController = m_rootViewController;

#pragma mark - Object creation

- (id)init
{
    self = [super init];
    if (self) {
        // application database (copy)
        [TTModel copyDatabaseIfNeed];
        
        // view for rootViewController
        TTPeopleViewController *peopleViewController = [[TTPeopleViewController alloc] init];
        
        // segmented control items
        NSArray *genderList = [TTGender findAll];
        NSMutableArray *segmentedItems = [NSMutableArray arrayWithCapacity:genderList.count];
        for (TTGender *item in genderList) {
            [segmentedItems addObject:NSLocalizedString(item.title, @"")];
            
        }
        
        // mount segmented control
        UISegmentedControl *segmentedControl = [[UISegmentedControl alloc] initWithItems:(NSArray *)segmentedItems];
        segmentedControl.tag = kGraderSegmentedControlTag;
        [segmentedControl setSegmentedControlStyle:UISegmentedControlStyleBar];
        segmentedControl.selectedSegmentIndex = 0;
        [segmentedControl addTarget:peopleViewController 
                             action:@selector(graderSegmentedControlValueChanged:) 
                   forControlEvents:UIControlEventValueChanged];
        
        CGRect segmentedControlFrame = segmentedControl.frame;
        segmentedControlFrame.size = CGSizeMake((90 * genderList.count), segmentedControlFrame.size.height);
        segmentedControl.frame = segmentedControlFrame;
        
        // set toolbar with centralized segmented control
        UIBarButtonItem *segmentedBarButton = [[UIBarButtonItem alloc] initWithCustomView:segmentedControl];
        UIBarButtonItem *flexibleSpaceBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace 
                                                                                                    target:peopleViewController 
                                                                                                    action:nil];
        
        NSArray *toolbarItems = [[NSArray alloc] initWithObjects:flexibleSpaceBarButtonItem, segmentedBarButton, flexibleSpaceBarButtonItem, nil];
        [segmentedControl release];
        [segmentedBarButton release];
        [flexibleSpaceBarButtonItem release];
        
        peopleViewController.toolbarItems = toolbarItems;
        [toolbarItems release];
        
        // root view controller navigation controller
        self.rootViewController = [[[UINavigationController alloc] initWithRootViewController:peopleViewController] autorelease];
        [peopleViewController release];
        [((UINavigationController *)self.rootViewController) setToolbarHidden:NO];
    }
    return self;
}

#pragma mark - Memory

- (void)dealloc
{
    self.rootViewController = nil;
    [super dealloc];
}

#pragma mark - Instance methods

- (UIViewController *)viewController
{
    return self.rootViewController;
}

@end
