//
//  TTPerson.m
//  Letra
//
//  Created by Erlon Cabral on 24/02/12.
//  Copyright 2012 aftertaste.cc. All rights reserved.
//

#import "TTPerson.h"

// category
#import "TTPerson+MapDatabaseResult.h"

// model
#import "TTGender.h"

NSString * const kFirstNameColunm = @"firstName";
NSString * const kLastNameColunm = @"lastName";


@implementation TTPerson

@synthesize firstName = _firstName;
@synthesize lastName =_lastName;
@synthesize genderKey = _genderKey;
@synthesize gender = _gender;

#pragma mark - 
#pragma mark - Instance methods

+ (TTPerson *)personWithFirstName:(NSString *)first lastName:(NSString *)last
{
    return [[[TTPerson alloc] initWithFirstName:first lastName:last] autorelease];
}

+ (TTPerson *)find:(NSInteger)primaryKey
{
	TTPerson *result = nil;
	EGODatabase *database = [EGODatabase databaseWithPath:[TTPerson dbPath]];
    
	if ([database open]){
		EGODatabaseResult *resultSet = [database executeQuery:@"SELECT * FROM person WHERE primaryKey = ? LIMIT 1" 
												   parameters:[NSArray arrayWithObjects:[NSNumber numberWithInt:primaryKey], nil]];
        
        result = [TTPerson personFromDatabaseResult:resultSet];
        [database close];
	}
	return result;
}

+ (NSArray *)findAll
{
	NSArray *result = nil;
	EGODatabase *database = [EGODatabase databaseWithPath:[TTPerson dbPath]];
	
	if ([database open]){
		EGODatabaseResult *resultSet = [database executeQuery:@"SELECT * FROM person ORDER BY firstName"];
        result = [TTPerson peopleFromDatabaseResult:resultSet];
        [database close];
	}
	return result;
}

+ (NSArray *)findAllByGenderKey:(NSInteger)genderKey
{
	NSArray *result = nil;
	EGODatabase *database = [EGODatabase databaseWithPath:[TTPerson dbPath]];
	
	if ([database open]){
		EGODatabaseResult *resultSet = [database executeQuery:@"SELECT * FROM person WHERE genderKey = ? ORDER BY firstName"
                                                   parameters:[NSArray arrayWithObjects:[NSNumber numberWithInt:genderKey], nil]];
        result = [TTPerson peopleFromDatabaseResult:resultSet];
        [database close];
	}
	return result;
}


#pragma mark - 
#pragma mark - Overrides

- (id)initWithFirstName:(NSString *)first lastName:(NSString *)last
{
    self = [super init];
    if (self) {
        self.firstName = first;
        self.lastName = last;
    }
    return self;
}

- (NSString *)description
{
	return [NSString stringWithFormat:@"<%@ primaryKey: %i, firstName: %@, lastName: %@>",
			NSStringFromClass([TTPerson class]), self.primaryKey, self.firstName, self.lastName];
}

#pragma mark - 
#pragma mark - Memory

- (void)dealloc
{
	[_firstName release];
	[_lastName release];
    [_gender release];
	[super dealloc];
}

#pragma mark - 
#pragma mark Instance methods

- (NSString *)fullName 
{
    return [[NSString stringWithFormat:@"%@ %@", 
             (self.firstName ? self.firstName : @""), 
             (self.lastName ? self.lastName: @"")] 
            stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
}

- (BOOL)validateFullName
{
    return ![[self fullName] isEqualToString:@""];
}

#pragma mark -
#pragma mark - Storage

- (BOOL)save
{
	BOOL success = NO;
	EGODatabase *database = [EGODatabase databaseWithPath:[TTPerson dbPath]];
	
	if ([database open]){
		success = [database executeUpdate:@"INSERT INTO person (firstName, lastName) VALUES (?, ?)" 
                               parameters:[NSArray arrayWithObjects:self.firstName, self.lastName, nil]];
		if (success) {
			self.primaryKey = [database last_insert_rowid];
		}
		[database close];
	}
	return success;
}

- (BOOL)update
{
	BOOL success = NO;
	EGODatabase *database = [EGODatabase databaseWithPath:[TTPerson dbPath]];
	
	if ([database open]){
		success = [database executeUpdate:@"UPDATE person SET firstName = ?, lastName = ? WHERE primaryKey = ?" 
							   parameters:[NSArray arrayWithObjects:self.firstName, self.lastName, [NSNumber numberWithInt:self.primaryKey], nil]];
		[database close];
	}
	return success;
}

- (BOOL)delete
{
	BOOL success = NO;
	EGODatabase *database = [EGODatabase databaseWithPath:[TTPerson dbPath]];
	
	if ([database open]){
		success = [database executeUpdate:@"DELETE * FROM person WHERE primaryKey = ?" 
							   parameters:[NSArray arrayWithObjects:[NSNumber numberWithInt:self.primaryKey], nil]];
		[database close];
	}
	return success;
}

- (void)loadGender
{
    if (self.genderKey > 0) {
        self.gender = [TTGender find:self.genderKey];
    }
}

@end
